classdef truss2dMinComplianceVolumeConstraint < truss2d 
    % This class encapsulates methods to minimize the compliance a 2d truss system
    % assumes that the truss members are circular rods
    % The design variables are the cross-sectional areas
    properties(GetAccess = 'public', SetAccess = 'private')
        myInitialVolume;
        myInitialArea;
        myInitialCompliance;
        myFinalArea;
        myFinalCompliance;
        myFinalVolume;
    end
    methods
        function obj = truss2dMinComplianceVolumeConstraint(xy,connectivity)
            obj = obj@truss2d(xy,connectivity);
        end
        function JRelative = complianceObjective(obj,x)
            Area = x.*obj.myInitialArea;
            obj = obj.assignA(Area);
            obj = obj.assemble();
            obj = obj.solve();
            J = obj.getCompliance();
            JRelative = J/obj.myInitialCompliance;
        end  
        function obj = assignYieldStress(obj,~,~)
            % dummy function
        end
        function obj = initialize(obj)
            obj.myInitialArea = obj.myArea;
            obj.myInitialVolume = sum(obj.myArea.*obj.myL);
            obj = obj.assemble();
            obj = obj.solve();
            obj.myInitialCompliance =  obj.getCompliance(); 
        end
        function obj = optimize(obj)
            % initialize quantities of interest
            obj = obj.initialize();
            x0 = ones(1,obj.myNumTrussBars); % unitless quantities 
            LB = 1e-12*ones(1,obj.myNumTrussBars); % small non-zero values
            AinEq = (obj.myInitialArea.*obj.myL)/obj.myInitialVolume;
            BinEq = 1; 
%             opt = optimset('fmincon');
%             opt = optimset(opt,'GradObj','off');
            [xMin,~,~,Output]  = fmincon(@obj.complianceObjective,x0, ...
                    AinEq,BinEq,[],[],LB);
            obj = obj.assignA(xMin.*obj.myInitialArea);
            obj = obj.assemble();
            obj = obj.solve();
            obj.myFinalArea = obj.myArea;
            obj.myFinalCompliance =  obj.getCompliance(); 
            obj.myFinalVolume = sum(obj.myArea.*obj.myL);
            disp(Output)
        end
    end
end

