% The SolidLabTestPart.sldprt must be open in SolidWorks 
% The Simulation in SolidWorks must be active 

%%Create solidLab object
format compact
s = solidLab();

%% mass properties
[volume,area,cg,inertia] = s.getMassProperties();
disp([' Volume = ' num2str(volume)]);

%% get all dimension names and values
[dimNames,dimValues] = s.getDimensions()


%% modify dimensions
s.modifyDimensions(dimNames([3,9]),[16,38]);

%% get volume
[volume,area,cg,inertia] = s.getMassProperties();
disp([' Volume = ' num2str(volume)]);


%% revert back the dimensions
s.modifyDimensions(dimNames([3,9]),[20,35]);
s.saveSolidWorksPart();
[volume,area,cg,inertia] = s.getMassProperties();
disp([' Volume = ' num2str(volume)]);
%% Solve FEA (must be in simulation mode)
s.createMesh();
s.solveStudy();
[maxVonMises,maxDisp] = s.getResults();
disp([' maxVonMises = ' num2str(maxVonMises)]);
disp([' maxDisp = ' num2str(maxDisp)]);

%% Parametric study 
nSamples = 10;
Displacement = zeros(nSamples,1);
Stress = zeros(nSamples,1);
pNormStress = zeros(nSamples,1);
Volume = zeros(nSamples,1);
dSamples = 15 + (25-15)*rand(nSamples,1); % diameter 
aSamples = 32 + (38-32)*rand(nSamples,1); % distance
for i = 1:nSamples
    i % to monitor progress
    d = dSamples(i);
    a = aSamples(i);
    s.modifyDimensions(dimNames([3,9]),[d,a]);
    s.saveSolidWorksPart();
    [volume,area,cg,inertia] = s.getMassProperties();
    s.createMesh();
    s.solveStudy();
    [maxVonMises,maxDisp] = s.getResults();
    
    Displacement(i) = maxDisp;
    Stress(i) = maxVonMises;
    Volume(i) = volume;
end
% reset to original values
s.modifyDimensions(dimNames([3,9]),[20,35]);
s.saveSolidWorksPart();
plot(Volume, Displacement,'*');
xlabel('Volume');
ylabel('Displacement');
figure;plot(Volume, Stress,'*');
xlabel('Volume');
ylabel('Stress');