classdef cantilever3DMinimizeDeflectionGS
    properties(GetAccess = 'public', SetAccess = 'private')
        mySolidLab;
        myInitialShapeParams; % used for scaling
        myUB; % upper bound on the shape parameters
        myLB;% lower bound on the shape parameters
        myInitialVolume; % used for scaling volume
        myInitialMaxDisplacement; % used for scaling compliance
        myNumElems;
        myFiniteDifferenceStepSize;
        myObjectiveTolerance;
        myParamTolerance;
    end
    methods
        function obj = cantilever3DMinimizeDeflectionGS(params0)
            obj.myInitialShapeParams = params0;
            obj.mySolidLab = solidLab(); % avoid initialization each time
            obj.myInitialMaxDisplacement = obj.getMaxDisplacementAtParams(params0);
            obj.myInitialVolume = obj.getVolumeAtParams(params0);
            obj.myFiniteDifferenceStepSize = 1e-3;
            obj.myObjectiveTolerance = 0.025;
            obj.myParamTolerance = 0.025;
        end
        function maxDisp = getMaxDisplacementAtParams(obj,params)
            obj.mySolidLab.modifyDimensions({'a@Sketch1@Cantilever.Part','b@Sketch1@Cantilever.Part'},params);
            obj.mySolidLab.createMesh();
            obj.mySolidLab.solveStudy();
            [~,maxDisp] = obj.mySolidLab.getResults();
        end
        function volume = getVolumeAtParams(obj,params)
            obj.mySolidLab.modifyDimensions({'a@Sketch1@Cantilever.Part','b@Sketch1@Cantilever.Part'},params);
            volume = obj.mySolidLab.getMassProperties();
        end
        function obj = setParamsUpperBound(obj,paramsUpper)
            obj.myUB = paramsUpper;
        end
        function obj = setObjectiveTolerance(obj,tol)
            obj.myObjectiveTolerance = tol;
        end
        function obj = setParamTolerance(obj,tol)
            obj.myParamTolerance = tol;
        end
        function obj = setFiniteDifferenceStepSize(obj,step)
            obj.myFiniteDifferenceStepSize = step;
        end
        function obj = setParamsLowerBound(obj,paramsLower)
            obj.myLB = paramsLower;
        end   
        function [f] = displacementObjective(obj,x)        
            params = x.*obj.myInitialShapeParams;
            maxDisp = obj.getMaxDisplacementAtParams(params);
            f = maxDisp/obj.myInitialMaxDisplacement; 
        end  
        function [cineq,ceq] = volumeConstraint(obj,x)
            params = x.*obj.myInitialShapeParams;
            volume = obj.getVolumeAtParams(params);
            ceq = [];
            cineq = volume/(obj.myInitialVolume) - 1.0;
        end
        function [results] = minimizeMaxDisplacement(obj) 
            LB = obj.myLB./obj.myInitialShapeParams;
            UB = obj.myUB./obj.myInitialShapeParams;
            x0 = ones(1,2); 
            opt = optimoptions('fmincon',...
            'Algorithm','SQP',...
            'TolX',obj.myParamTolerance,...
            'TolFun',obj.myObjectiveTolerance,...
            'PlotFcns',@optimplotfval,...
            'FiniteDifferenceStepSize',obj.myFiniteDifferenceStepSize);   

            problem = createOptimProblem('fmincon','objective', @obj.displacementObjective, ...
                 'x0', x0, 'lb', LB,'ub', UB, 'nonlcon',@obj.volumeConstraint,'options',opt);
      
            GS = GlobalSearch('MaxTime',600);
            [xMin,fMin,flag,output] = run(GS,problem);
            results.initialParams =  obj.myInitialShapeParams;
            results.initialVolume =  obj.myInitialVolume;
            results.initialMaxDisplacement =  obj.myInitialMaxDisplacement;
            results.flag = flag;
            results.finalParams = xMin.*obj.myInitialShapeParams;
            results.finalVolume  = obj.getVolumeAtParams(results.finalParams);
            results.finalMaxDisplacement = fMin*obj.myInitialMaxDisplacement;
            % each function call involves additional FEAs per parameter
            results.nFEAs = output.funcCount*(1+length(obj.myInitialShapeParams));
        end
    end
end

