classdef solidLabTestPart3DMinimizeStress
    properties(GetAccess = 'public', SetAccess = 'private')
        mySolidLab;
        myInitialShapeParams; % used for scaling
        myUB; % upper bound on the shape parameters
        myLB;% lower bound on the shape parameters
        myInitialVolume; % used for scaling volume
        myInitialMaxStress; 
        myFinalMaxStress; 
        myParamTolerance;
        myObjectiveTolerance;
        myMaxFunctionEvaluations;
    end
    methods
        function obj = solidLabTestPart3DMinimizeStress(params0)
            obj.myInitialShapeParams = params0;
            obj.mySolidLab = solidLab(); % avoid initialization each time
            obj.myInitialVolume = obj.getVolumeAtParams(params0);
            obj.myInitialMaxStress = obj.getMaxStressAtParams(params0);
            obj.myMaxFunctionEvaluations = 100;
            obj.myParamTolerance = 1e-3;
            obj.myObjectiveTolerance = 1e-3;
        end
         function maxStress = getMaxStressAtParams(obj,params)
            obj.mySolidLab.modifyDimensions({'a@SideHolesSketch@solidLabTestPart.Part',...
                'd@SideHolesSketch@solidLabTestPart.Part'},params);
            obj.mySolidLab.createMesh();
            obj.mySolidLab.solveStudy();
            [maxStress] = obj.mySolidLab.getResults();
            
        end
        function volume = getVolumeAtParams(obj,params)
            obj.mySolidLab.modifyDimensions({'a@Sketch1@Cantilever.Part','b@Sketch1@Cantilever.Part'},params);
            volume = obj.mySolidLab.getMassProperties();
        end
        function obj = setParamsUpperBound(obj,paramsUpper)
            obj.myUB = paramsUpper;
        end
        function obj = setParamTolerance(obj,tol)
            obj.myParamTolerance = tol;
        end
        function obj = setObjectiveTolerance(obj,tol)
            obj.myObjectiveTolerance = tol;
        end
        function obj = setFiniteDifferenceStepSize(obj,step)
            obj.myFiniteDifferenceStepSize = step;
        end
        function obj = setParamsLowerBound(obj,paramsLower)
            obj.myLB = paramsLower;
        end   
        function [f] = stressObjective(obj,x)        
            params = x.*obj.myInitialShapeParams;
            maxStress = obj.getMaxStressAtParams(params);
            f = maxStress/obj.myInitialMaxStress;
        end  
        function [cineq,ceq] = volumeConstraint(obj,x)
            params = x.*obj.myInitialShapeParams;
            volume = obj.getVolumeAtParams(params);
            ceq = [];
            cineq = volume/(obj.myInitialVolume) - 1.0;
        end
        function [results] = minimizeMaxStress(obj) 
            LB = obj.myLB./obj.myInitialShapeParams;
            UB = obj.myUB./obj.myInitialShapeParams;
            x0 = ones(1,2); 
            opt = optimoptions('fmincon',...
            'TolFun',obj.myObjectiveTolerance,...
            'TolX',obj.myParamTolerance,...
            'MaxFunEvals',obj.myMaxFunctionEvaluations,...
            'PlotFcns',@optimplotfval,...
            'Display','iter');
            [xMin,~,flag,output] = fminsearchcon(@(x)obj.stressObjective(x),x0,LB,UB,[],[],@(x)obj.volumeConstraint(x));
 
            results.initialParams =  obj.myInitialShapeParams;
            results.initialVolume =  obj.myInitialVolume;
            results.flag = flag;
            results.finalParams = xMin.*obj.myInitialShapeParams;
            results.finalVolume  = obj.getVolumeAtParams(results.finalParams);
            results.initialMaxStress = obj.myInitialMaxStress;
            results.finalMaxStress = obj.getMaxStressAtParams(results.finalParams);
            % each function call involves additional FEAs per parameter
            results.nFEAs = output.funcCount;
        end
    end
end

