classdef PolynomialClass
    properties(GetAccess = 'public', SetAccess = 'private')
        % public read access, but private write access.
        myCoefficients;
    end
    methods
        function obj = PolynomialClass(a)
            % constructor for a PolynomialClass
            % a is a vector
            obj.myCoefficients = a;
        end
        function y = evaluate(obj,x)
            % evaluate the PolynomialClass at x
            % y = a1 + a2*x + a3*x^2 + ...
            y = 0;
            for i = 1:length(obj.myCoefficients)
                y = y + obj.myCoefficients(i)*(x^(i-1));
            end
        end
        function string = char(obj)
            % return a string representation of a polynomial 
            string = '';
            for i = 1:length(obj.myCoefficients)
                coeff = obj.myCoefficients(i);
                if coeff == 0
                    continue
                elseif (coeff < 0)
                    sign = '-';
                else
                    sign = '+';
                end
                % handle the first two coefficients as a special case
                if (i == 1)
                    if (coeff < 0)
                        string = [string sign num2str(abs(coeff))];
                    else
                        string = [string sign num2str(abs(coeff))];
                    end
                elseif (i == 2)
                    string = [string sign num2str(abs(coeff)) '*x'];
                else
                    string = [string sign num2str(abs(coeff)) '*x^' num2str(i-1)];
                end
            end
            if (string(1) == '+') % remove the initial '+' sign
                string = string(2:end);
            end
        end
        function disp(obj)
            % Pretty print of a PolynomialClass object
            disp(char(obj))
        end
        function r = plus(obj1,obj2)
            % add two PolynomialClasss; the two PolynomialClasss may be of different order
            if (isnumeric(obj1)) % if scalar, convert to polynomial
                obj1 = PolynomialClass(obj1);
            end
            if (isnumeric(obj2)) % if scalar, convert to polynomial
                obj2 = PolynomialClass(obj2);
            end
            % Find the difference in order
            n1 = length(obj1.myCoefficients);
            n2 = length(obj2.myCoefficients);
            if (n1 > n2) % append zeros to p2, and then add
                r = PolynomialClass(obj1.myCoefficients + [obj2.myCoefficients zeros(1,n1-n2)]);
            elseif (n1 < n2) % append zeros to p1, and then add
                r = PolynomialClass([ obj1.myCoefficients zeros(1,n2-n1)] + obj2.myCoefficients);
            else % just add coefficients
                r = PolynomialClass(obj1.myCoefficients + obj2.myCoefficients);
            end
        end
        
    end
end
