classdef testFunctions1D
    % This class encapsulates numerous 1-D test functions
    methods(Static)
        function [f,g,h] = f1(x)
            % returns  (x-1)^2 and its derivatives
            f = (x-1).^2;
            if (nargout > 1), g = 2*(x-1);end
            if (nargout > 2), h = 2;end
        end
        function [f,g,h] = f2(x)
            % returns (x-1)^3 and its derivatives
            f = (x-1).^3;
            if (nargout > 1), g = 3*(x-1).^2;end
            if (nargout > 2), h = 6*(x-1);end
        end
        function [f,g,h] = f3(x)
            % returns (x-1)^4 and its derivatives
            f = (x-1).^4;
            if (nargout > 1), g = 4*(x-1).^3;end
            if (nargout > 2), h = 12*(x-1).^2;end
        end
        function [f] = f4(x)
            % returns abs(x-1)
            f = abs(x-1);
        end
        function [f,g] = f5(x)
            % if (x < 1) f = 2*(x-1)^2 else f = 4*(x-1)^2
            f = (x<1).*(2*(x-1).^2) + (x>=1).*(4*(x-1).^2);
            if (nargout > 1), g = (x<1).*(4*(x-1)) + (x>=1).*(8*(x-1));end
        end
        function [f,g,h] = f6(x)
            % returns cos^2(x) and its derivatives
            f = cos(x).^2;
            if (nargout > 1), g = -2*cos(x).*sin(x);end
            if (nargout > 2), h = 2*sin(x).^2 - 2*cos(x).^2;end
        end
        function [f,g,h] = f7(x)
            % returns  1-x*exp(-x^2) and optionally its derivatives
            f = (1-x.*exp(-x.^2));
            if (nargout > 1), g = -exp(-x.^2) + 2*(x.^2).*exp(-x.^2);end
            if (nargout > 2), h = 6*x.*exp(-x.*x)- 4*x.^3.*exp(-x.*x);end
        end 
        function [g,h] = g7(x)
            % returns gradient of 1-x*exp(-x^2) 
            g = -exp(-x.^2) + 2*(x.^2).*exp(-x.^2);
            if (nargout > 1), h = 6*x.*exp(-x.*x)- 4*x.^3.*exp(-x.*x);end
        end
        function [f] = f7Noisy(x,noise)
            if (nargin == 1), noise = 0.1; end
            % returns  1-x*exp(-x^2) with noise
            f = (1-x.*exp(-x.^2)) + noise*sin(100*x);
        end 
        function [f,g,h] = f8(x)
            % returns  3*sin(x)-x+0.1*x.^2 +0.1*cos(2*x) and its derivatives
            f = 3*sin(x)-x+0.1*x.^2 +0.1*cos(2*x);
            if (nargout > 1), g = 3*cos(x)-1+0.2*x-0.2*sin(2*x); end
            if (nargout > 2), h = -3*sin(x)+0.2-0.4*cos(2*x);end
        end 
        function [g,h] = g8(x)
            % returns  gradient of 3*sin(x)-x+0.1*x.^2 +0.1*cos(2*x)    
            g = 3*cos(x)-1+0.2*x-0.2*sin(2*x);
            if (nargout > 2), h = -3*sin(x)+0.2-0.4*cos(2*x);end
        end 
        function [f,g] = f9(x,a)
            if (nargin == 1)
                a = 0.025;
            end
            f = (x-1).^2 + a*(sin(1000*(x-1)/pi)).^2;
            if (nargout > 1), g = 2*(x-1) + a*2*(1000/pi)*cos(1000*(x-1)/pi).*sin(1000*(x-1)/pi); end
        end 
        function [f,g,h] = f10(x)
            % sin(x) + sin(10x/3)
            f = sin(x)+sin(10*x/3);
            if (nargout > 1), g = cos(x) + (10/3)*cos(10*x/3); end
            if (nargout > 2), h = -sin(x) -(10/3)^2*sin(10*x/3); end
        end 
        function [g] = g10(x)
            %gradient of sin(x) + sin(10x/3)
            g = cos(x) + (10/3)*cos(10*x/3);
        end 
        function [f,g,h] = f11(x)
            % -(1.4 - 3x)sin(18x)
            f = -(1.4 - 3*x).*sin(18*x);
            if (nargout > 1), g = 3*sin(18*x) + 18*cos(18*x).*(3*x - 7/5); end
            if (nargout > 2), h = 108*cos(18*x) - 324*sin(18*x).*(3*x - 7/5); end
        end 
        function [g] = g11(x)
            %gradient of -(1.4 - 3x)sin(18x)
            g = 3*sin(18*x) + 18*cos(18*x).*(3*x - 7/5);
        end 
    end
end
