classdef equalityConstrainedProblems
    % This class encapsulates a few equality constrained problems
    methods(Static)
        function [f,g] = sphericalObjective3D(xVec)
            x = xVec(1);y = xVec(2);z = xVec(3);
            f = (x-3).^4 + (y-1).^2 + z.^2;
            if (nargout > 1), g = [4*(x-3).^3; 2*(y-1); 2*z];end
        end
        function f = thomsonProblemObjective(pts)
            % Potential energy of nP electrons
            % pts: 3 x N
            nP = size(pts,2);
            f = 0;
            for i = 1:nP-1
                for j = i+1:nP
                    f = f + 1/norm(pts(:,i) - pts(:,j));
                end 
            end
        end
        function [c,ceq] = thomsonProblemConstraint(pts)
            % Constraint on nP electrons
            % pts: 3 x N
            nP = size(pts,2);
            c = [];
            ceq = zeros(nP,1);
            for i = 1:nP
               ceq(i) = norm(pts(:,i))^2 -1;
            end
        end
        function [c,ceq] = ParabolicConstraint2D(x)
            c = [];
            ceq = x(2) - x(1)^2;
        end
        function [c,ceq] = CircleConstraint2D(x)
            c = [];
            ceq = x(1)^2 + x(2)^2 - 1;
        end
    end
end
