classdef twoSpringWithConstraints < spring2dPlotter
    methods 
        function obj = twoSpringWithConstraints()
            % two-spring nodes and connectivity
            nodeXY = [0 0; 0 -1; 0 1]'; % 2 x nNodes
            connectivity = [1 2; 1 3]'; % 2 x nSprings
            obj = obj@spring2dPlotter(nodeXY,connectivity); % call superclass
        end
        function [PE,gradient_f] = potentialEnergy(obj,uv) %#ok<INUSL>
            % returns the potential energy of a 2-spring system
            fx1 = 10;
            fy1 = 8;
            k1 = 100; % stiffness of spring 1
            k2 = 50;% stiffness of spring 2
            u = uv(1);v = uv(2);          
            L12 = sqrt(u^2+(1+v)^2); % deformed lengths
            L13 = sqrt(u^2+(1-v)^2);
            deltaL12 = (L12 -1);
            deltaL13 = (L13 -1);
            U = 0.5*k1*deltaL12.^2 + 0.5*k2*deltaL13.^2; % Elastic energy
            W = 0.5*(fx1*u+fy1*v); % Work done
            PE =  U-2*W; % potential energy
            if (nargout == 2)
                gradient_f(1) = k1*(L12-1)/L12*u + k2*(L13-1)/L13*u - fx1;
                gradient_f(2) = k1*(L12-1)/L12*(1+v) + k2*(L13-1)/L13*(v-1) - fy1;
            end
        end
        function [residual] = forceBalance(~,uv)
            fx1 = 10;
            fy1 = 8;
            k1 = 100; % stiffness of spring 1
            k2 = 50;% stiffness of spring 2
            u = uv(1);v = uv(2);
            L12 = sqrt(u^2+(1+v)^2); % deformed lengths
            L13 = sqrt(u^2+(1-v)^2);
            residual(1) = k1*(L12-1)/L12*u + k2*(L13-1)/L13*u - fx1;
            residual(2) = k1*(L12-1)/L12*(1+v) + k2*(L13-1)/L13*(v-1) - fy1;
        end
        function [cineq,ceq,grad_cineq, grad_ceq] = constraintsEquality(~,uv)
            u = uv(1);v = uv(2);
            cineq = [];
            ceq(1) = u.^3 -v  ;
            grad_cineq = [];
            grad_ceq = [3*u.^2; -1];
        end
        function [cineq,ceq,grad_cineq, grad_ceq] = constraintsInequality(~,uv)
            u = uv(1);v = uv(2);
            ceq = [];
            cineq(1) = -v + u.^3 ;
            grad_ceq = [];
            grad_cineq = [3*u.^2; -1];
        end
    end
end
